tnc

package
v4.0.16 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Aug 23, 2026 License: BSD-3-Clause Imports: 8 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func AllTNCs

func AllTNCs() []string

AllTNCs returns an alphabetical list of the names of all supported TNCs.

func TNCDir

func TNCDir() string

TNCDir returns the pathname of the directory that contains TNC definitions.

Types

type TNC

type TNC struct {
	// Name is the name of the TNC type.
	Name string
	// CommandPrompt is the TNC command prompt that we wait for between
	// commands.
	CommandPrompt string
	// ConnectedMessage is the message from the TNC that indicates it is
	// connected to a BBS.
	ConnectedMessage string
	// DisconnectedMessage is the message from the TNC that indicates it has
	// lost its connection to the BBS.
	DisconnectedMessage string
	// PreConnectCommands is the list of commands sent to the TNC prior to
	// establishing a connection.  Note this does not include MyCallCommand.
	PreConnectCommands []string
	// PostDisconnectCommands is the list of commands sent to the TNC after
	// disconnecting from a BBS.
	PostDisconnectCommands []string
	// MyCallCommand is the command to set the call sign used for
	// connections.
	MyCallCommand string
	// ConnectCommand is the command to establish a connection.
	ConnectCommand string
	// EchoTimeout is the number of milliseconds to wait for the TNC to echo
	// data sent to it.
	EchoTimeout int
	// ReplyTimeout is the number of milliseconds to wait for the TNC to
	// reply to commands sent to it (not involving RF).
	ReplyTimeout int
}

A TNC describes the details of interacting with a particular type of TNC.

func Get

func Get(name string) *TNC

Get returns the TNC definition for the TNC with the specified name, or nil if there is none.

Jump to